/* wad.c contains the main function for the wad (work-around daemon)
   whose primary function is to adjust tcp parameters to optimum
   values for connections listed in the wad.conf file.  
   The daemon will be notified via a PF_NETLINK socket when a
   web100 tcp connection has started or stopped when run as root.
   Otherwise, it will poll /proc/web100 checking for new connections.
*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/errno.h>
#include <signal.h>
#include <asm/types.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <sys/select.h>
#include <sys/time.h>
#include "web100.h"

extern void read_config();
extern void TuneConnections(struct web100_agent *, int);
extern void WalkProcWeb100(struct web100_agent *);
extern struct web100_agent *wad_init();

main (int argc, char *argv[]) {
  struct web100_agent *agent;
  int sd, lth, err;
  double buff[1024];
  struct sockaddr_nl nl;
  int cid_nl;
  double naptime = 0.1;
  int n, napsec=0, i=0;
  struct timeval tv;
  struct Msg {
    int type;
    int cid;
  }*msg;

/* kill -1 pid   will send the process the signal SIGHUP to reload
   the connections to be adjusted from wad.conf */
  signal(SIGHUP, read_config);
  if(argc > 1) {
    naptime = atof(argv[1]);
    if(naptime <= 0.0)
      naptime = 0.1;
    else
      if(naptime >=1.0) {
        napsec = (int)naptime;
        naptime = naptime - napsec;
      }
  }

  agent=wad_init();

#ifdef NETLINK_WEB100
/* wait for connection notification and take the
   appropriate action */
    if((sd = socket(PF_NETLINK, SOCK_RAW, NETLINK_WEB100)) < 0) {
      perror("socket");
      exit(1);
    }
    bzero(&nl, sizeof(nl));
    nl.nl_family = AF_NETLINK;
    nl.nl_groups = ~0;  /* all groups */
    if((bind(sd,(struct sockaddr *) &nl, sizeof(nl))) < 0) {
      perror("netlink socket bind");
      close(sd);
      fprintf(stderr, "So...POLLING /proc/web100!\n");
      while(1) {
        WalkProcWeb100(agent);
        tv.tv_sec = napsec;
        tv.tv_usec = (naptime * 1000000);
        n = select((int)NULL, NULL, NULL, NULL, &tv);
      }
    }
    msg = (struct Msg *)buff;
fprintf(stderr, "Using NETLINK!\n");
    while(1) {
      if((lth = read(sd, buff, sizeof(buff))) < 0) {
        if(errno == EINTR)
	  continue;
	else {
          perror("Netlink Socket read");
          exit(1);
	}
      }
      if(msg->type == 0) {
        if(msg->cid > 0) 
          TuneConnections(agent, msg->cid);
      }
      else {
	if(msg->cid > 0) 
	  cid_nl = msg->cid;
	  err = DeleteCid(cid_nl);
      } 
    }
#else
    fprintf(stderr, "POLLING /proc/web100\n");
    while(1) {
      WalkProcWeb100(agent);
      tv.tv_sec = napsec;
      tv.tv_usec = (naptime * 1000000);
      n = select((int)NULL, NULL, NULL, NULL, &tv);
    } 
#endif
}
